import { useEffect, useState } from "react";
import {
  Constants,
  FormatDatetime,
  myFetch,
  wsConnectionCustomEventName,
} from "../../utils";
import { useCustomerFeedbackContext } from "../../Contexts/CustomerFeedbackContext";
import { Card, Col, Result, Row, Select, Spin, Statistic, Table } from "antd";
import { useTranslation } from "react-i18next";
import { RequestState } from "../../Components/MyRequestState";
import { useNavigate, useParams } from "react-router-dom";
import CountUp from "react-countup";
import {
  LineChart,
  Line,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";

function calculateAverage(arr) {
  if (arr.length === 0) return 0;
  const sum = arr.reduce(
    (accumulator, currentValue) => accumulator + currentValue,
    0
  );
  const average = sum / arr.length;

  return average.toFixed(1);
}

export default function CustomerFeedback() {
  const { t } = useTranslation();
  const navigate = useNavigate();
  const customerFeedbackContext = useCustomerFeedbackContext();
  const { paramOrigin } = useParams();

  const [isRequesting, setIsRequesting] = useState(RequestState.INIT);
  const [selectedOrigin, setSelectedOrigin] = useState("");

  const getTableContent = () => {
    const feedbackColumns = [];
    const columns = [];

    for (const feedback of customerFeedbackContext.customerFeedbacks) {
      if (feedback.Data === null) continue;

      for (const key in feedback.Data) {
        if (!columns.includes(key)) {
          feedbackColumns.push({
            title: key,
            dataIndex: key,
            key: key,
          });

          columns.push(key);
        }
      }
    }

    return [
      ...feedbackColumns,
      {
        title: t("customerFeedback.table.createdAt"),
        dataIndex: "createdAt",
        key: "createdAt",
      },
    ];
  };

  const getTableItems = () => {
    const items = [];

    customerFeedbackContext.customerFeedbacks.sort(
      (a, b) => new Date(b.CreatedAt) - new Date(a.CreatedAt)
    );

    customerFeedbackContext.customerFeedbacks.forEach((customerFeedback) => {
      let dynamicItems = {};

      if (customerFeedback.Data !== null) {
        for (const key in customerFeedback.Data) {
          dynamicItems[key] = customerFeedback.Data[key];
        }
      }

      items.push({
        key: customerFeedback.Id,
        ...dynamicItems,
        createdAt: FormatDatetime(customerFeedback.CreatedAt),
      });
    });

    return items;
  };

  const navigateToOrigin = (origin) =>
    navigate(`${Constants.ROUTE_PATHS.CUSTOMERFEEDBACK_VIEW}/${origin}`);

  const StatisticCard = () => {
    const data = {};

    for (const customerFeedback of customerFeedbackContext.customerFeedbacks) {
      if (customerFeedback.Data === null) continue;

      for (const feedbackDataKey in customerFeedback.Data) {
        if (typeof customerFeedback.Data[feedbackDataKey] !== "number")
          continue;

        if (!data.hasOwnProperty(feedbackDataKey)) {
          data[feedbackDataKey] = [];
        }

        data[feedbackDataKey].push(customerFeedback.Data[feedbackDataKey]);
      }
    }

    const elements = [
      <Statistic
        title={t("customerFeedback.feedbacks")}
        value={customerFeedbackContext.customerFeedbacks.length}
        formatter={(value) => <CountUp end={value} />}
      />,
    ];

    for (const statistic in data) {
      elements.push(
        <Statistic
          key={statistic}
          title={statistic}
          value={calculateAverage(data[statistic])}
          formatter={(value) => (
            <span>
              <CountUp end={value} prefix="Ø " decimals={1} /> / ∑{" "}
              {data[statistic].length}
            </span>
          )}
        />
      );
    }

    const chunkedElements = elements.reduce((acc, element, index) => {
      const chunkIndex = Math.floor(index / 8);
      if (!acc[chunkIndex]) {
        acc[chunkIndex] = [];
      }
      acc[chunkIndex].push(element);
      return acc;
    }, []);

    return (
      <Card style={{ marginTop: 20 }}>
        {chunkedElements.map((chunk, chunkIndex) => (
          <Row gutter={16} key={chunkIndex}>
            {chunk.map((element, elementIndex) => (
              <Col xs={24} sm={12} md={4} key={elementIndex}>
                {element}
              </Col>
            ))}
          </Row>
        ))}
      </Card>
    );
  };

  const Graphs = () => {
    if (
      !customerFeedbackContext.customerFeedbacks ||
      customerFeedbackContext.customerFeedbacks.length === 0
    ) {
      return <div>No data available</div>;
    }

    // Function to count number of feedbacks for each date
    const countFeedbacksByDate = (feedbacks) => {
      const dateCounts = {};

      // Iterate through feedbacks
      feedbacks.forEach((feedback) => {
        const date = new Date(feedback.CreatedAt).toISOString().split("T")[0];

        // Increment count for date or initialize it with 1
        dateCounts[date] = (dateCounts[date] || 0) + 1;
      });

      // Convert date-counts map to array of objects and sort by date ascending
      return Object.keys(dateCounts)
        .map((date) => ({
          date,
          count: dateCounts[date],
        }))
        .sort((a, b) => new Date(a.date) - new Date(b.date));
    };

    const data = countFeedbacksByDate(
      customerFeedbackContext.customerFeedbacks
    );

    // Function to extract data for each key in Data
    const getDataByKey = (feedbacks, key) => {
      const dateValuesMap = {};

      // Iterate through feedbacks
      feedbacks.forEach((feedback) => {
        if (feedback.Data && typeof feedback.Data === "object") {
          const date = new Date(feedback.CreatedAt).toISOString().split("T")[0];
          const value = feedback.Data[key];

          // Increment count for date or initialize it with value
          dateValuesMap[date] =
            (dateValuesMap[date] || 0) + (typeof value === "number" ? 1 : 0);
        }
      });

      // Convert date-values map to array of objects and sort by date ascending
      return Object.keys(dateValuesMap)
        .map((date) => ({
          date,
          value: dateValuesMap[date],
        }))
        .sort((a, b) => new Date(a.date) - new Date(b.date));
    };

    return (
      <div>
        <Card style={{ marginTop: 20 }}>
          <h2>Feedback Count</h2>
          <ResponsiveContainer width="100%" height={400}>
            <LineChart data={data}>
              <CartesianGrid strokeDasharray="3 3" />
              <XAxis dataKey="date" />
              <YAxis />
              <Tooltip />
              <Legend />
              <Line
                type="monotone"
                dataKey="count"
                stroke="#8884d8"
                activeDot={{ r: 8 }}
              />
            </LineChart>
          </ResponsiveContainer>
        </Card>
      </div>
    );

    /*
  {Object.keys(customerFeedbackContext.customerFeedbacks[0]?.Data || {})
          .filter(
            (key) =>
              typeof customerFeedbackContext.customerFeedbacks[0]?.Data[key] ===
              "number"
          )
          .map((key) => (
            <Card key={key} style={{ marginTop: 20 }}>
              <h2>{key}</h2>
              <ResponsiveContainer width="100%" height={400}>
                <LineChart
                  data={getDataByKey(
                    customerFeedbackContext.customerFeedbacks,
                    key
                  )}
                >
                  <CartesianGrid strokeDasharray="3 3" />
                  <XAxis dataKey="date" />
                  <YAxis />
                  <Tooltip />
                  <Legend />
                  <Line
                    type="monotone"
                    dataKey="value"
                    stroke="#8884d8"
                    activeDot={{ r: 8 }}
                  />
                </LineChart>
              </ResponsiveContainer>
            </Card>
          ))}
    */
  };

  // Function to group data by date and count occurrences
  /*const getDataByDate = (feedbacks) => {
      const dateCounts = feedbacks.reduce((acc, feedback) => {
        const date = new Date(feedback.CreatedAt).toISOString().split("T")[0]; // Extract date part
        acc[date] = (acc[date] || 0) + 1;
        return acc;
      }, {});

      const sortedDates = Object.keys(dateCounts).sort(
        (a, b) => new Date(a) - new Date(b)
      );

      return sortedDates.map((date) => ({
        date,
        count: dateCounts[date],
      }));
    };

    const data = getDataByDate(customerFeedbackContext.customerFeedbacks);

    return (
      <Card style={{ marginTop: 20 }}>
        <ResponsiveContainer width="100%" height={400}>
          <LineChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="date" />
            <YAxis />
            <Tooltip />
            <Legend />
            <Line
              type="monotone"
              dataKey="count"
              stroke="#8884d8"
              activeDot={{ r: 8 }}
            />
          </LineChart>
        </ResponsiveContainer>
      </Card>
    );
  }; */

  useEffect(() => {
    const customersRequest = () => {
      setIsRequesting(RequestState.REQUESTING);

      myFetch(`/customerfeedback/origins`, "GET").then((data) => {
        if (data === undefined || data === null) return;

        customerFeedbackContext.setOrigins(data);

        if (data.length === 0) {
          setIsRequesting(RequestState.NOTHING);
        } else if (paramOrigin === undefined || !data.includes(paramOrigin)) {
          navigateToOrigin(data[0]);
        }
      });
    };

    customersRequest();

    const handleCustomersRequest = () => customersRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleCustomersRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleCustomersRequest
      );
  }, []);

  useEffect(() => {
    if (paramOrigin === undefined) return;

    setIsRequesting(RequestState.REQUESTING);
    setSelectedOrigin(paramOrigin);

    myFetch(`/customerfeedback/origin/${paramOrigin}`, "GET").then((data) => {
      if (data === undefined || data === null) return;

      customerFeedbackContext.setCustomerFeedbacks(data);
      setIsRequesting(RequestState.SUCCESS);
    });
  }, [paramOrigin]);

  return (
    <>
      <div
        style={{
          display: "flex",
          flexDirection: "row",
          gap: 10,
          alignItems: "center",
        }}
      >
        <span>{t("customerFeedback.origin")}</span>

        <Select
          style={{
            width: "100%",
          }}
          value={selectedOrigin}
          onSelect={(value) => navigateToOrigin(value)}
          options={customerFeedbackContext.origins.map((origin) => {
            return {
              value: origin,
              label: origin,
            };
          })}
        ></Select>
      </div>

      {isRequesting === RequestState.INIT ? (
        <div
          style={{
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            height: "80vh",
          }}
        >
          <Spin size="large"></Spin>
        </div>
      ) : isRequesting === RequestState.NOTHING ? (
        <Result
          status="404"
          title="404"
          subTitle="Sorry, the page you visited does not exist."
        />
      ) : (
        <>
          <StatisticCard />

          <Graphs />

          <Table
            style={{ paddingTop: 20 }}
            scroll={{ x: "max-content" }}
            columns={getTableContent()}
            dataSource={getTableItems()}
            loading={isRequesting === RequestState.REQUESTING}
            pagination
          />
        </>
      )}
    </>
  );
}
